#include "kernel/types.h"
#include "kernel/stat.h"
#include "user/user.h"

int global;

// messages in the order they will appear when run
char msg_parent_no_writes[] = "\nparent process after fork and before anyone has written";
char msg_child_no_writes[] =
    "\nchild process after fork and before anyone has written\n"
    "user pages should be same as parent, but page table pages different";

char msg_parent_after_first_heap_write[] =
    "\nparent process after writing to heap for first time\n"
    "heap page should be new and writable";

char msg_child_before_first_heap_write[] =
    "\nchild process before writing to heap for first time\n"
    "heap page should be same as before and not writable";

char msg_child_after_first_heap_write[] =
    "\nchild process after writing to heap for first time\n"
    "heap page should be same as before but writable now";

char msg_child_after_first_stack_write[] =
    "\nchild process after writing to stack for first time\n"
    "stack page should be new and writable";

char msg_parent_after_first_stack_write[] =
    "\nparent process after writing to stack for first time\n"
    "stack page should be same as before but writable now";

char msg_child_trying_to_write_to_text[] =
    "\nchild process about to try to write to text segment\n"
    "should trap and be killed\n";

char msg_parent_exiting[] =
    "\nparent exiting normally\n";

int
main(int argc, char **argv)
{
    volatile int mypid = 0;
    printf("Forking with cowfork...\n");

    int newpid = cowfork();
    if (newpid < 0) {
        printf("cowfork error\n");
        exit(1);
    }

    if (newpid != 0) {
        // parent
        write(1, msg_parent_no_writes, sizeof(msg_parent_no_writes)-1);
        pages(getpid());

        sleep(4);

        global = 5;
        write(1, msg_parent_after_first_heap_write, sizeof(msg_parent_after_first_heap_write)-1);
        pages(getpid());

        sleep(4);

        mypid = getpid();
        write(1, msg_parent_after_first_stack_write, sizeof(msg_parent_after_first_stack_write)-1);
        pages(mypid);

        sleep(4);

        write(1, msg_parent_exiting, sizeof(msg_parent_exiting)-1);
    } else {
        // child
        sleep(2);

        write(1, msg_child_no_writes, sizeof(msg_child_no_writes)-1);
        pages(getpid());

        sleep(4);

        write(1, msg_child_before_first_heap_write, sizeof(msg_child_before_first_heap_write)-1);
        pages(getpid());
        global = 5;
        write(1, msg_child_after_first_heap_write, sizeof(msg_child_after_first_heap_write)-1);
        pages(getpid());

        mypid = getpid();
        write(1, msg_child_after_first_stack_write, sizeof(msg_child_after_first_stack_write)-1);
        pages(mypid);

        sleep(4);

        write(1, msg_child_trying_to_write_to_text, sizeof(msg_child_trying_to_write_to_text)-1);
        *((int *) main) = 10;
    }

    exit(0);
}
